@@ -475,7 +475,7 @@ static void draw_plist_time ()
 	sec_to_min_plist (buf, time);
 	wmove (info_win, 0, COLS - 12);
 	waddch (info_win, all_files ? ' ' : '>');
-	wprintw (info_win, "%09s", buf);
+	wprintw (info_win, "%s", buf);
 }
 
 /* Draw the entry. Use this function at the end of screen drawing, because
@@ -985,7 +985,7 @@ static void fill_times (struct plist *plist, struct menu *menu)
 
 			read_item_time (&plist->items[i]);
 
-			if (menu && get_item_time(plist, i)) {
+			if (menu && get_item_time(plist, i) != -1) {
 				char time_str[6];
 				sec_to_min (time_str, get_item_time(plist, i));
 				menu_item_set_time_plist (menu, i, time_str);
@@ -2043,6 +2043,9 @@ static void event_plist_add (struct plist_item *item)
 				toggle_plist ();
 			waiting_for_plist_load = 0;
 		}
+		else if (visible_plist == playlist)
+			update_curr_file (); /* toggle_plist() above already
+						does it. */
 
 	}
 }
